"use strict";

/* PopUpSelector.js
 *
 * Author: Alexander.Trofimov@avsmedia.net
 * Date:   April 7, 2014
 */
(	/**
 	 * @param {jQuery} $
 	 * @param {Window} window
 	 * @param {undefined} undefined
 	 */
	function($, window, undefined) {
		var asc		= window["Asc"];
		var asc_HL	= asc.HandlersList;

		function PopUpSelector(element, handlers) {
			this.handlers		= new asc_HL(handlers);

			this.element		= element;
			this.selector		= null;
			this.selectorStyle	= null;
			this.selectorList	= null;
			this.selectorListEl	= [];
			this.selectorListJQ	= null;
			this.selectElement	= null;
			this.firstElement	= null;

			this.isFormula		= false;
			this.isVisible		= false;

			this.skipClose		= false;

			this.fMouseDown		= null;
			this.fMouseDblClick	= null;
			this.fMouseOver		= null;

			this._init();
			return this;
		}
		PopUpSelector.prototype._init = function () {
			var t = this;
			if (null != this.element) {
				this.selector = document.createElement('div');
				this.selectorStyle = this.selector.style;
				this.selector.id = 'apiPopUpSelector';
				this.selector.innerHTML = '<div style="max-height:210px;" class="combobox"><ul id="apiPopUpList" class="dropdown-menu"></ul></div>';

				this.element.appendChild(this.selector);
				this.selectorList = document.getElementById("apiPopUpList");

				this.fMouseDown = function (event) {t._onMouseDown(event);};
				this.fMouseDblClick = function (event) {t._onMouseDblClick(event);};
				this.fMouseOver = function (event) {t._onMouseOver(event);};

				if (this.selector.addEventListener) {
					this.selector.addEventListener("mousedown", function () {
						t.skipClose = true;
					}, false);
				}
				if (window.addEventListener) {
					window.addEventListener("mousedown", function () {
						if (t.skipClose) {
							t.skipClose = false;
							return;
						}
						t.hide();
					}, false);
				}

				// Для того, чтобы работал scroll
				this.selectorListJQ = $('#apiPopUpList');
				this.selectorListJQ.perfectScrollbar();
			}
		};
		PopUpSelector.prototype.show = function (isFormula, arrItems, cellRect) {
			this._clearList();
			this.setPosition(cellRect);
			if (!this.isVisible) {
				this.selectorStyle.display = "block";
				this.isVisible = true;
			}
			this.isFormula = isFormula;

			var item, isFirst, value;
			for (var i = 0; i < arrItems.length; ++i) {
				item = document.createElement("li");
				isFirst = (0 === i);
				if (isFirst)
					this.firstElement = item;

				if (this.isFormula) {
					if (isFirst) {
						this.selectElement = item;
						item.className = "selected";
					}

					value = arrItems[i].name;
					item.setAttribute("title", arrItems[i].arg);
				} else
					value = arrItems[i];

				item.innerHTML = "<a>" + value + "</a>";
				item.setAttribute("val", value);

				if (item.addEventListener) {
					item.addEventListener("mousedown"	, this.fMouseDown		, false);
					item.addEventListener("dblclick"	, this.fMouseDblClick	, false);
					if (!this.isFormula) {
						item.addEventListener("mouseover", this.fMouseOver, false);
					}
				}

				this.selectorList.appendChild(item);
				this.selectorListEl.push(item);
			}

			// Selection hack
			/*var clearSelection = function() {
				if (document.selection && document.selection.empty) {
					document.selection.empty();
				}
				else if (window.getSelection) {
					var sel = window.getSelection();
					sel.removeAllRanges();
				}
			}*/
			// TODO: В Mozilla избавиться от селекта текста при dblclick

			// Для того, чтобы работал scroll
			this.selectorListJQ.perfectScrollbar("update");

		};
		PopUpSelector.prototype.hide = function () {
			if (this.isVisible) {
				this.selectorStyle.display = "none";
				this.isVisible = false;

				this._clearList();
			}
		};
		PopUpSelector.prototype.setPosition = function (cellRect) {
			this.selectorStyle["left"] = (cellRect.asc_getX() + 10) + "px";
			this.selectorStyle["top"] = (cellRect.asc_getY() + cellRect.asc_getHeight()) + "px";
		};
		PopUpSelector.prototype.getVisible = function () {
			return this.isVisible;
		};
		PopUpSelector.prototype._clearList = function () {
			var elem;
			while(elem = this.selectorListEl.pop())
				this.selectorList.removeChild(elem);

			this.selectElement = null;
			this.firstElement = null;
			this.isFormula = false;
		};

		PopUpSelector.prototype.onKeyDown = function (event) {
			var retVal = false;
			switch (event.which) {
				case 9: // Tab
					if (this.isFormula)
						this._onMouseDblClick();
					else
						retVal = true;
					break;
				case 13:  // "enter"
					if (!this.isFormula && null !== this.selectElement)
						this._onInsert(this.selectElement.getAttribute("val"));
					else
						retVal = true;
					break;
				case 27: // Esc
					this.hide();
					break;
				case 38: // Up
					this._onChangeSelection(null !== this.selectElement ?
						this.selectElement.previousSibling : this.firstElement, -1);
					break;
				case 40: // Down
					this._onChangeSelection(null !== this.selectElement ?
						this.selectElement.nextSibling : this.firstElement, +1);
					break;
				case 16: // Shift
					break;
				default:
					retVal = true;
			}
			if (retVal)
				this.hide();
			return retVal;
		};

		PopUpSelector.prototype._onInsert = function (value) {
			this.hide();
			this.handlers.trigger("insert", value);
		};

		PopUpSelector.prototype._onMouseDown = function (event) {
			this.skipClose = true;
			var element = event.currentTarget;
			if (this.isFormula) {
				this._onChangeSelection(element);
			} else {
				this._onInsert(element.getAttribute("val"));
			}
		};
		PopUpSelector.prototype._onMouseDblClick = function (event) {
			if (!this.isVisible)
				return;

			if (!this.isFormula) {
				this._onMouseDown(event);
				return;
			}
			var elementVal = (event ? event.currentTarget : this.selectElement).getAttribute("val") + "(";
			this._onInsert(elementVal);
		};
		PopUpSelector.prototype._onMouseOver = function (event) {
			if (this.isFormula)
				return;

			this._onChangeSelection(event.currentTarget);
		};
		PopUpSelector.prototype._onChangeSelection = function (newElement, scrollDir) {
			if (null === newElement)
				return;

			var height = 0;
			if (null !== this.selectElement) {
				height = this.selectElement.offsetHeight;
				this.selectElement.className = "";
			}

			this.selectElement = newElement;
			this.selectElement.className = "selected";

			if (scrollDir && height)
				this.selectorListJQ.scrollTop(scrollDir * height);
		};

		/*
		 * Export
		 * -----------------------------------------------------------------------------
		 */
		window["Asc"].PopUpSelector = PopUpSelector;
	}
)(jQuery, window);